#include <stdio.h>
#include <libxml/parser.h>
#include <assert.h>

#include <GDD/arch_xml.h>
#include <GDD/ddg_xml.h>
#include <RS/rs.h>
#include <RS/greedy.h>
#include <RS/vsa.h>

int main(int argc, char *argv[]) {
  // initialise XML parser
  xmlInitParser();

  // read architecture
  GDD_Arch *arch = GDD_arch_of_xml_file("arch.xml");
  assert(arch != NULL);
      
  // read ddg (ignore interloop dependency)
  SCEDA_Graph *dag = GDD_of_xml_file(TRUE, arch, "ddg.xml",NULL);
  assert(dag != NULL);

  // clean up XML parser
  xmlCleanupParser();

  GDD_DAG *ddg = GDD_dag_create(dag); 
  assert(ddg != NULL);
  
  fprintf(stdout,"estimated GR saturation = %d\n",RS_estimate_saturation(ddg, "GR", FALSE));

  SCEDA_HashMap *k = RS_ddag_greedy_k(ddg, "GR");

  GDD_Opcode *dummy;
  SCEDA_Graph *ddg_k = RS_ensure_saturating_schedule(ddg, "GR", k, &dummy);

  fprintf(stdout,"estimated GR saturation = %d\n",RS_estimate_saturation_TC(ddg_k, "GR", FALSE));

  GDD_opcode_delete(dummy);

  SCEDA_graph_delete(ddg_k);

  SCEDA_hashmap_delete(k);

  GDD_dag_delete(ddg); // clear the DDAG

  SCEDA_graph_delete(dag);
  GDD_arch_delete(arch);
      
  return 0;
}
